@@ -1,7 +1,5 @@
-use io;
-use std::vec::Vec;
-use std::string::String;
-use byteorder::{LittleEndian, ByteOrder};
+use crate::rust::{vec::Vec, string::String};
+use crate::{io, elements};
 use super::{Error, Deserialize, Serialize};
 
 /// Unsigned variable-length integer, limited to 32 bits,
@@ -29,7 +27,7 @@ impl From<u32> for VarUint32 {
 
 impl From<usize> for VarUint32 {
 	fn from(i: usize) -> VarUint32 {
-		assert!(i <= ::std::u32::MAX as usize);
+		assert!(i <= u32::max_value() as usize);
 		VarUint32(i as u32)
 	}
 }
@@ -417,7 +415,7 @@ impl Deserialize for Uint32 {
 		let mut buf = [0u8; 4];
 		reader.read(&mut buf)?;
 		// todo check range
-		Ok(Uint32(LittleEndian::read_u32(&buf)))
+		Ok(u32::from_le_bytes(buf).into())
 	}
 }
 
@@ -431,9 +429,7 @@ impl Serialize for Uint32 {
 	type Error = Error;
 
 	fn serialize<W: io::Write>(self, writer: &mut W) -> Result<(), Self::Error> {
-		let mut buf = [0u8; 4];
-		LittleEndian::write_u32(&mut buf, self.0);
-		writer.write(&buf)?;
+		writer.write(&self.0.to_le_bytes())?;
 		Ok(())
 	}
 }
@@ -453,7 +449,7 @@ impl Deserialize for Uint64 {
 		let mut buf = [0u8; 8];
 		reader.read(&mut buf)?;
 		// todo check range
-		Ok(Uint64(LittleEndian::read_u64(&buf)))
+		Ok(u64::from_le_bytes(buf).into())
 	}
 }
 
@@ -461,9 +457,7 @@ impl Serialize for Uint64 {
 	type Error = Error;
 
 	fn serialize<W: io::Write>(self, writer: &mut W) -> Result<(), Self::Error> {
-		let mut buf = [0u8; 8];
-		LittleEndian::write_u64(&mut buf, self.0);
-		writer.write(&buf)?;
+		writer.write(&self.0.to_le_bytes())?;
 		Ok(())
 	}
 }
@@ -605,9 +599,9 @@ impl<'a, W: 'a + io::Write> io::Write for CountedWriter<'a, W> {
 /// Helper struct to write series of `T` preceded by the length of the sequence
 /// serialized as VarUint32.
 #[derive(Debug, Clone)]
-pub struct CountedListWriter<I: Serialize<Error=::elements::Error>, T: IntoIterator<Item=I>>(pub usize, pub T);
+pub struct CountedListWriter<I: Serialize<Error=elements::Error>, T: IntoIterator<Item=I>>(pub usize, pub T);
 
-impl<I: Serialize<Error=::elements::Error>, T: IntoIterator<Item=I>> Serialize for CountedListWriter<I, T> {
+impl<I: Serialize<Error=elements::Error>, T: IntoIterator<Item=I>> Serialize for CountedListWriter<I, T> {
 	type Error = Error;
 
 	fn serialize<W: io::Write>(self, writer: &mut W) -> Result<(), Self::Error> {
@@ -627,7 +621,7 @@ mod tests {
 
 	use super::super::{deserialize_buffer, Serialize};
 	use super::{CountedList, VarInt7, VarUint32, VarInt32, VarInt64, VarUint64};
-	use elements::Error;
+	use crate::elements::Error;
 
 	fn varuint32_ser_test(val: u32, expected: Vec<u8>) {
 		let mut buf = Vec::new();
