@@ -1,7 +1,5 @@
-use io;
-use std::vec::Vec;
-use std::string::String;
-use std::borrow::ToOwned;
+use crate::rust::{vec::Vec, string::String, borrow::ToOwned};
+use crate::{io, elements};
 use super::{
 	Serialize,
 	Deserialize,
@@ -60,6 +58,8 @@ pub enum Section {
 	Start(u32),
 	/// Elements section.
 	Element(ElementSection),
+	/// Number of passive data entries in the data section
+	DataCount(u32),
 	/// Function bodies section.
 	Code(CodeSection),
 	/// Data definition section.
@@ -127,6 +127,12 @@ impl Deserialize for Section {
 				11 => {
 					Section::Data(DataSection::deserialize(reader)?)
 				},
+				12 => {
+					let mut section_reader = SectionReader::new(reader)?;
+					let count = VarUint32::deserialize(&mut section_reader)?;
+					section_reader.close()?;
+					Section::DataCount(count.into())
+				},
 				invalid_id => {
 					return Err(Error::InvalidSectionId(invalid_id))
 				},
@@ -182,6 +188,12 @@ impl Serialize for Section {
 				VarUint32::from(index).serialize(&mut counted_writer)?;
 				counted_writer.done()?;
 			},
+			Section::DataCount(count) => {
+				VarUint7::from(0x0c).serialize(writer)?;
+				let mut counted_writer = CountedWriter::new(writer);
+				VarUint32::from(count).serialize(&mut counted_writer)?;
+				counted_writer.done()?;
+			},
 			Section::Element(element_section) => {
 				VarUint7::from(0x09).serialize(writer)?;
 				element_section.serialize(writer)?;
@@ -212,7 +224,7 @@ impl Serialize for Section {
 }
 
 impl Section {
-	pub(crate) fn id(&self) -> u8 {
+	pub(crate) fn order(&self) -> u8 {
 		match *self {
 			Section::Custom(_) => 0x00,
 			Section::Unparsed { .. } => 0x00,
@@ -225,8 +237,9 @@ impl Section {
 			Section::Export(_) => 0x7,
 			Section::Start(_) => 0x8,
 			Section::Element(_) => 0x9,
-			Section::Code(_) => 0x0a,
-			Section::Data(_) => 0x0b,
+			Section::DataCount(_) => 0x0a,
+			Section::Code(_) => 0x0b,
+			Section::Data(_) => 0x0c,
 			Section::Name(_) => 0x00,
 			Section::Reloc(_) => 0x00,
 		}
@@ -239,7 +252,7 @@ pub(crate) struct SectionReader {
 }
 
 impl SectionReader {
-	pub fn new<R: io::Read>(reader: &mut R) -> Result<Self, ::elements::Error> {
+	pub fn new<R: io::Read>(reader: &mut R) -> Result<Self, elements::Error> {
 		let length = u32::from(VarUint32::deserialize(reader)?) as usize;
 		let inner_buffer = buffered_read!(ENTRIES_BUFFER_LENGTH, length, reader);
 		let buf_length = inner_buffer.len();
@@ -270,8 +283,8 @@ impl io::Read for SectionReader {
 	}
 }
 
-fn read_entries<R: io::Read, T: Deserialize<Error=::elements::Error>>(reader: &mut R)
-	-> Result<Vec<T>, ::elements::Error>
+fn read_entries<R: io::Read, T: Deserialize<Error=elements::Error>>(reader: &mut R)
+	-> Result<Vec<T>, elements::Error>
 {
 	let mut section_reader = SectionReader::new(reader)?;
 	let result = CountedList::<T>::deserialize(&mut section_reader)?.into_inner();
@@ -297,7 +310,7 @@ impl CustomSection {
 		&self.name
 	}
 
-	/// Payload of the custom secion.
+	/// Payload of the custom section.
 	pub fn payload(&self) -> &[u8] {
 		&self.payload
 	}
