@@ -1,14 +1,10 @@
 //! WebAssembly format library
+#![warn(missing_docs)]
 
 #![cfg_attr(not(feature = "std"), no_std)]
 #![cfg_attr(not(feature = "std"), feature(alloc))]
 
-#![warn(missing_docs)]
-
-extern crate byteorder;
-
-#[cfg(not(feature = "std"))]
-#[macro_use]
+#[cfg(not(feature="std"))] #[macro_use]
 extern crate alloc;
 
 pub mod elements;
@@ -28,13 +24,17 @@ pub use elements::{
 	serialize_to_file,
 };
 
-
-
 #[cfg(not(feature = "std"))]
-mod std {
+pub (crate) mod rust {
 	pub use core::*;
-	pub use alloc::vec;
-	pub use alloc::string;
-	pub use alloc::boxed;
-	pub use alloc::borrow;
+	pub use ::alloc::format;
+	pub use ::alloc::vec;
+	pub use ::alloc::string;
+	pub use ::alloc::boxed;
+	pub use ::alloc::borrow;
 }
+
+#[cfg(feature="std")]
+pub (crate) mod rust {
+	pub use std::*;
+}
\ No newline at end of file
